<?php

namespace Drupal\amu_hal\Plugin\Block;

use Drupal\Core\Block\BlockBase;
use Drupal\Core\Block\BlockPluginInterface;
use Drupal\Core\Form\FormStateInterface;
use GuzzleHttp\Client;
use Drupal\user\Entity\User;
use GuzzleHttp\Exception\ConnectException;
use Drupal\Core\Url;


/**
 * Provides a 'HalPublicationBlock' Block.
 *
 * Récupère les paramètres de de configuration du bloc et génère l'url de
 * requete du WS HAL
 *
 * @Block(
 *   id = "HalPublicationBlock",
 *   admin_label = @Translation("Hal publications"),
 * )
 */
class HalPublicationBlock extends BlockBase implements BlockPluginInterface {

  /**
   * {@inheritdoc}
   */
  public function build() {
    //retrieve module global parameters
    $module_global_config = \Drupal::configFactory()
      ->getEditable('amu_hal.settings');
    $amu_hal_url_ws = $module_global_config->get('amu_hal_url_ws');
    $hal_date_start = $module_global_config->get('hal_date_start');
    $docType_s = $module_global_config->get('docType_s');

    //retrieve bloc parameters
    $config = $this->getConfiguration();
    $isDisplay = 'block-display-true';

    $year_param = \Drupal::request()->query->get('year') ?: NULL;
    if ($year_param) {
      $config['year_filter'] = $year_param;
      $config['call-to-actions-widget_select'] = 'year_navigation';
    }

    $row_max_unset = \Drupal::request()->query->get('row_max_unset') ?: NULL;
    if ($row_max_unset) {
      $config['last_publications_max_number_display'] = 2000;
      $config['year_filter'] = (int) date('Y');
      $config['call-to-actions-widget_select'] = 'year_navigation';
    }

    $doctypes = '';
    $array_filtered=array_filter($config['doctype_filter']);
    foreach ( $array_filtered as $doctype) {
      if (end($array_filtered ) != $doctype) {
        $doctypes .= $doctype . ' OR ';
      }
      else {
        $doctypes .= $doctype;
      }
    }

    $url = '';
    switch ($config['retrieval_method_choice']) {
      case 'by_halId_s':
        $url = $amu_hal_url_ws . 'search/' . $config['halId_s'] . '?rows=' . $config['last_publications_max_number_display'] . '&sort=producedDate_tdate+desc&fq=docType_s:(' . $doctypes . ')';
        break;
      case 'by_structId_i':
        $url = $amu_hal_url_ws . 'search/?rows=' . $config['last_publications_max_number_display'] . '&sort=producedDate_tdate+desc&fq=structId_i:' . $config['structId_i'] . '+docType_s:(' . $doctypes . ')';
        break;
      case 'by_user_fields':
        $url = $amu_hal_url_ws;
        $current_path = \Drupal::service('path.current')->getPath();
        $exploded_path = explode('/', $current_path);
        $uid = end($exploded_path);
        if (prev($exploded_path) == 'user') {
          $user = User::load($uid);
          if ($user) {
            if ($user->hasField('field_publications_block_select')) {
              if ($user->get('field_publications_block_select')
                  ->getString() == 'selected publications') {
                $isDisplay = 'block-display-none';
              }
            }
            //le dernier prevaut
            if ($user->hasField('field_cn_ldap')) {
              $authLastNameFirstName_s =strtolower($user->get('field_cn_ldap')
                  ->getString()) ;
              $url = $amu_hal_url_ws . 'search/?rows=' . $config['last_publications_max_number_display'] . '&sort=producedDate_tdate+desc&fq=authLastNameFirstName_s:("' . ucwords($authLastNameFirstName_s) . '")';
            }
            if ($user->hasField('field_formes_auteur')) {
              $tabFormesAuteur = $user->get('field_formes_auteur')->getString();
              if ($tabFormesAuteur && '' != $tabFormesAuteur) {
                $tabFormesAuteur = preg_split("/\\r\\n|\\r|\\n/", $tabFormesAuteur);
                array_walk_recursive($tabFormesAuteur, function (&$value) {
                  $value = ucwords(strtolower($value));
                });
                $authLastNameFirstName_s = '"';
                $authLastNameFirstName_s .= implode('" OR "', $tabFormesAuteur);
                $authLastNameFirstName_s .= '"';
                $url = $amu_hal_url_ws . 'search/?rows=' . $config['last_publications_max_number_display'] . '&sort=producedDate_tdate+desc&fq=authLastNameFirstName_s:(' . $authLastNameFirstName_s . ')';
              }
            }
            if ($user->hasField('field_authidhal_i')) {
              $authIdHal_I = $user->get('field_authidhal_i')->getString();
              if ($authIdHal_I && '' != $authIdHal_I) {
                $url = $amu_hal_url_ws . 'search/?rows=' . $config['last_publications_max_number_display'] . '&sort=producedDate_tdate+desc&fq=authIdHal_i:' . $authIdHal_I . '';
              }
            }
            if ($user->hasField('field_docids')) {
              $docids = $user->get('field_docids')->getString();
              if ($docids && '' != $docids) {
                $tabdocids = preg_split("/\\r\\n|\\r|\\n/", $docids);
                $end = end($tabdocids);
                $url = $amu_hal_url_ws . 'search/?rows=' . $config['last_publications_max_number_display'] . '&fq=docid:(';
                foreach ($tabdocids as $docid) {
                  if ($end != $docid) {
                    $url .= $docid . ' OR ';
                  }
                  else {
                    $url .= $docid . ')';
                  }
                }
              }
            }
          }
        }
        break;
      case 'by_docid':
        $tabdocids = preg_split("/\\r\\n|\\r|\\n/", $config['docid']);
        $end = end($tabdocids);
        $url = $amu_hal_url_ws . 'search/?rows=' . $config['last_publications_max_number_display'] . '&fq=docid:(';
        foreach ($tabdocids as $docid) {
          if ($end != $docid) {
            $url .= $docid . ' OR ';
          }
          else {
            $url .= $docid . ')';
          }
        }
        $url .= '+docType_s:(' . $doctypes . ')';
        break;
      default:
        $url = $amu_hal_url_ws;
        break;
    }
    if (NULL != $config['year_filter']) {
      $url .= '&fq=producedDateY_i:' . $config['year_filter'];
    }

    // filtre par mots clés
    if (isset($_POST['recherche_sur_titre']) && NULL != $_POST['recherche_sur_titre']) {
      $keywords = '(' . $_POST['recherche_sur_titre'] . ')';
      $url .= '&q=title_t:' . urlencode($keywords);
    }
    //filtre par auteur
    if (isset($_POST['recherche_par_auteur']) && NULL != $_POST['recherche_par_auteur']) {
      $filtre_auteurs = '("' . $_POST['recherche_par_auteur'] . '")';
      $url .= '&q=authLastName_t:' . urlencode($filtre_auteurs);
    }

    $url .= '&fl=title_s,en_title_s,docid,label_s,en_label_s,docType_s,authIdHal_s,halId_s,structId_i,uri_s,keyword_s,en_keyword_s,authLastNameFirstName_s,journalTitle_s,abstract_s,en_abstract_s,producedDate_tdate,producedDateY_i,language_s,fileMain_s';

    try {
      $client = new Client();
      $response = $client->request('GET', $url);
      $content = json_decode($response->getBody(), TRUE);
      // var_dump($content);
    } catch (ConnectException $e) {
      \Drupal::logger('amu_hal')->error('cette url n est pas bonne' . $url);
    }

    $year_tab = [];
    $current_date = (int) date('Y');
    $selected_year=(isset($_GET['year']))? (int)$_GET['year']:$current_date;

    $internal_link=null;
    switch ($config['call-to-actions-widget_select']) {
      case 'year_navigation':
        $row_max_unset = FALSE;
        do {
          $year_tab[] = $current_date;
          $current_date--;
        } while ($current_date != $hal_date_start);
        break;
      case 'row_max_unset':
        $row_max_unset = TRUE;
        break;
      case 'internal_link':
        $internal_link = $config['internal_link'];
        break;
    }

    //le nom du twig file name defined in .module amu_hal_theme
    if ($config['display_mode_select'] == 'academic_view') {
      $theme = 'block-AcademicDisplay';
    }
    else {
      $theme = 'block-FancyDisplay';
    }
    $skin = 'amu_hal_skin_' . $config['skin_select'];

    $cache = [
      'max-age' => 0,
    ];

    $keywords_search_inputs = array_filter($config['keywords_search']);
    $title_search = array_key_exists('title_search', $keywords_search_inputs);
    $author_search = array_key_exists('author_search', $keywords_search_inputs);

    return [
      '#cache' => $cache,
      '#theme' => $theme,
      "#content" => $content,
      '#url' => $url,
      '#docTypes_array' => $docType_s,
      '#title_search' => $title_search,
      '#author_search' => $author_search,
      '#year_tab' => $year_tab,
      '#selected_year' => $selected_year,
      '#row_max_unset' => $row_max_unset,
      '#internal_link' => $internal_link,
      '#isDisplay' => $isDisplay,
      '#attached' => [
        'library' => [
          'amu_hal/amu_hal_library',
          'amu_hal/' . $skin,
        ],
      ],
    ];
  }

  /**
   * {@inheritdoc}
   *
   * custom parameters for instances of block
   */
  public function blockForm($form, FormStateInterface $form_state) {

    $form = parent::blockForm($form, $form_state);
    $module_global_config = \Drupal::configFactory()
      ->getEditable('amu_hal.settings');
    $config = $this->getConfiguration();

    $url = Url::fromRoute('help.main');
    $internal_link = \Drupal::l(t('Accès à la documentation du module HAL'), $url);
    $link_doctype = Url::fromUri('https://api.archives-ouvertes.fr/ref/doctype');
    $external_link = \Drupal::l(t('See the list of HAL doc types'), $link_doctype);

    $form['retrieval_method'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Méthode d\'importation'),
    ];

    $form['retrieval_method']['retrieval_method_choice'] = [
      '#type' => 'select',
      '#title' => $this->t('Choose between:'),
      '#description' => $internal_link,
      '#default_value' => (!empty($config['retrieval_method_choice'])) ? $config['retrieval_method_choice'] : 'by_halId_s',
      '#options' => [
        'by_halId_s' => $this->t('Publications from a collection'),
        'by_structId_i' => $this->t('Publications from a structure'),
        'by_docid' => $this->t('Defined list of publications'),
        'by_user_fields' => $this->t('Publications by author ( on profile page )'),
      ],
    ];

    $form['retrieval_method']['halId_s'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Collection HAL ID'),
      '#description' => $this->t('The ID of the collection is the HAL halId_s'),
      '#default_value' => (!empty($config['halId_s'])) ? $config['halId_s'] : $module_global_config->get('halId_s'),
      '#states' => [
        'visible' => [
          [':input[name="settings[retrieval_method][retrieval_method_choice]"]' => ['value' => 'by_halId_s']],
          'or',
          [':input[name="settings[retrieval_method][retrieval_method_choice]"]' => ['value' => 'by_authLastNameFirstName_s']],
        ],
      ],
    ];

    $form['retrieval_method']['structId_i'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Structure HAL ID'),
      '#description' => $this->t('The ID of the structure is the HAL structId_i'),
      '#default_value' => (!empty($config['structId_i'])) ? $config['structId_i'] : $module_global_config->get('structId_i'),
      '#states' => [
        'visible' => [
          [':input[name="settings[retrieval_method][retrieval_method_choice]"]' => ['value' => 'by_structId_i']],
          'or',
          [':input[name="settings[retrieval_method][retrieval_method_choice]"]' => ['value' => 'by_authLastNameFirstName_s']],
        ],
      ],
    ];

    $form['retrieval_method']['authIdHal_i'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Author HAL ID'),
      '#description' => $this->t('The ID of the author is the HAL authIdHal_i'),
      '#default_value' => (!empty($config['authIdHal_i'])) ? $config['authIdHal_i'] : '',
      '#states' => [
        'visible' => [
          ':input[name="settings[retrieval_method][retrieval_method_choice]"]' => ['value' => 'by_authIdHal_i'],
        ],
      ],
    ];

    $form['retrieval_method']['authLastNameFirstName_s'] = [
      '#type' => 'textarea',
      '#title' => $this->t('HAL author Forme-Auteur(s)'),
      '#description' => $this->t('Indicate the Form_Auteur(s), one on each lines as name surname'),
      '#default_value' => (!empty($config['authLastNameFirstName_s'])) ? $config['authLastNameFirstName_s'] : '',
      '#states' => [
        'visible' => [
          ':input[name="settings[retrieval_method][retrieval_method_choice]"]' => ['value' => 'by_authLastNameFirstName_s'],
        ],
      ],
    ];

    $form['retrieval_method']['docid'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Document HAL ID'),
      '#description' => $this->t('List all docids, one per line'),
      '#default_value' => (!empty($config['docid'])) ? $config['docid'] : '',
      '#states' => [
        'visible' => [
          ':input[name="settings[retrieval_method][retrieval_method_choice]"]' => ['value' => 'by_docid'],
        ],
      ],
    ];

    $form['sorting_options'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('sorting options'),
    ];

    $form['sorting_options']['sorting_options choices'] = [
      '#type' => 'select',
      '#title' => $this->t('sorting options'),
      '#description' => $this->t(''),
      '#options' => [
        'last publications' => $this->t('Most recent publications'),
      ],
      '#default_value' => (!empty($config['sorting_options choices'])) ? $config['sorting_options choices'] : 'last publications',
    ];

    $form['sorting_options']['last_publications_max_number_display'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Publications max display number'),
      '#description' => $this->t(''),
      '#default_value' => (!empty($config['last_publications_max_number_display'])) ? $config['last_publications_max_number_display'] : 5,
    ];

    $form['filter_options'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Filter options'),
    ];

    $form['filter_options']['keywords_search'] = [
      '#type' => 'checkboxes',
      '#title' => $this->t('Adding keywords search'),
      '#description' => $this->t(''),
      '#default_value' => (!empty($config['keywords_search'])) ? $config['keywords_search'] : NULL,
      '#options' => [
        'title_search' => $this->t('search by title'),
        'author_search' => $this->t('search by author'),
      ],
    ];

    $form['filter_options']['year_filter'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Filter by year'),
      '#description' => $this->t('ex:2014'),
      '#default_value' => (!empty($config['year_filter'])) ? $config['year_filter'] : NULL,
    ];

    $form['filter_options']['doctype_filter'] = [
      '#type' => 'checkboxes',
      '#title' => $this->t('filter by doctypes'),
      '#description' => $external_link,
      '#default_value' => (!empty($config['doctype_filter'])) ? $config['doctype_filter'] : [
        "ART",
        "COMM",
        "POSTER",
        "OUV",
        "DOUV",
        "PATENT",
        "OTHER",
      ],
      '#options' => [
        'ART' => $this->t('ART'),
        'COMM' => $this->t('COMM'),
        'POSTER' => $this->t('POSTER'),
        'OUV' => $this->t('OUV'),
        'DOUV' => $this->t('DOUV'),
        'PATENT' => $this->t('PATENT'),
        'OTHER' => $this->t('OTHER'),
      ],
    ];

    $form['call-to-actions-widget'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Call-to-Action widgets'),
    ];

    $form['call-to-actions-widget']['call-to-actions-widget_select'] = [
      '#type' => 'select',
      '#title' => $this->t('widget options'),
      '#description' => $this->t('Choose a widget'),
      '#options' => [
        'none' => $this->t('none'),
        'year_navigation' => $this->t('year exposed filter'),
        'row_max_unset' => $this->t('more publications'),
        'internal_link' => $this->t('more publications ( internal link )'),
      ],
      '#default_value' => (!empty($config['call-to-actions-widget_select'])) ? $config['call-to-actions-widget_select'] : 'none',
    ];

    $form['call-to-actions-widget']['internal_link'] = [
      '#type' => 'textfield',
      '#title' => $this->t('internal link'),
      '#description' => $this->t('link url. Example: /node/56'),
      '#default_value' => (!empty($config['internal_link'])) ? $config['internal_link'] : '',
      '#states' => [
        'visible' => [
          ':input[name="settings[call-to-actions-widget][call-to-actions-widget_select]"]' => ['value' => 'internal_link'],
        ],
      ],
    ];

    $form['display_mode'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('display mode'),
    ];
    $form['display_mode']['display_mode_select'] = [
      '#type' => 'select',
      '#title' => $this->t('display mode'),
      '#description' => $this->t('Choose between possible options'),
      '#options' => [
        'academic_view' => $this->t('academic view'),
        'fancy_view' => $this->t('fancy view'),
      ],
      '#default_value' => (!empty($config['display_mode_select'])) ? $config['display_mode_select'] : 'academic view',
    ];

    $form['skin'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('skin'),
    ];
    $form['skin']['skin_select'] = [
      '#type' => 'select',
      '#title' => $this->t('skin'),
      '#description' => $this->t('Choose between possible options'),
      '#options' => [
        'generic' => $this->t('neutral'),
        'pink_power' => $this->t('pink power'),
      ],
      '#default_value' => (!empty($config['skin_select'])) ? $config['skin_select'] : 'neutral',
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function blockSubmit($form, FormStateInterface $form_state) {
    $form_state_values = $form_state->getValues();

    $this->configuration['retrieval_method_choice'] = $form_state_values['retrieval_method']['retrieval_method_choice'];
    $this->configuration['halId_s'] = $form_state_values['retrieval_method']['halId_s'];
    $this->configuration['structId_i'] = $form_state_values['retrieval_method']['structId_i'];
    $this->configuration['docid'] = $form_state_values['retrieval_method']['docid'];
    $this->configuration['sorting_options choices'] = $form_state_values['sorting_options']['sorting_options choices'];
    $this->configuration['keywords_search'] = $form_state_values['filter_options']['keywords_search'];
    $this->configuration['year_filter'] = $form_state_values['filter_options']['year_filter'];
    $this->configuration['doctype_filter'] = $form_state_values['filter_options']['doctype_filter'];
    $this->configuration['last_publications_max_number_display'] = $form_state_values['sorting_options']['last_publications_max_number_display'];
    $this->configuration['call-to-actions-widget_select'] = $form_state_values['call-to-actions-widget']['call-to-actions-widget_select'];
    $this->configuration['internal_link'] = $form_state_values['call-to-actions-widget']['internal_link'];
    $this->configuration['display_mode_select'] = $form_state_values['display_mode']['display_mode_select'];
    $this->configuration['skin_select'] = $form_state_values['skin']['skin_select'];
  }
}
